package com.qmrz.generator;

import com.qmrz.exception.ABException;
import com.qmrz.service.TableService;
import com.qmrz.utils.CGHelper;
import com.qmrz.utils.Fn;
import com.qmrz.utils.SpringContextUtil;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Mapper生成
 */
public class JavaSyncGenerator {
    private TableService tableService;
    private String dbName;

    public JavaSyncGenerator(String dbName) {
        tableService = SpringContextUtil.getBean(TableService.class);
        this.dbName = dbName;
    }

    public void generator() {
        List<Map> tableList = tableService.getTableList(dbName);
        List<Map> columnList = tableService.getColumnList(dbName);

        Map<String, List<Map>> result = columnList.stream().collect(Collectors.groupingBy(map -> Fn.toString(map.get("table_name")), Collectors.toList()));

//        CGHelper cgHelper = new CGHelper("mapper2.ftl", dbName);//无同步代码版本
        CGHelper cgHelper = new CGHelper("java_sync.ftl", dbName);//有同步代码版本

        tableList.forEach(item -> {
            String table_name = item.get("table_name").toString();
            String table_name2 = Fn.firstUpperCase(table_name);

            List<Map> tmpColumnList = result.get(table_name);
            StringBuilder primaryKeyName = new StringBuilder();

            tmpColumnList.stream().filter(i -> {
                if ("PRI".equals(Fn.toString(i.get("column_key")))) {
                    primaryKeyName.append(Fn.toString(i.get("column_name")));
                    return true;
                }
                return false;
            }).findFirst();

            if (primaryKeyName.length() == 0) {
                throw new ABException("表 " + table_name + " 没有主键");
            }

            tmpColumnList = tmpColumnList.stream()
                    .filter(i ->
                            !"syscreatetime".equals(Fn.toString(i.get("column_name")))
                            && !"sysupdatetime".equals(Fn.toString(i.get("column_name")))
                    ).collect(Collectors.toList());

            item.put("columnList", tmpColumnList);
            item.put("dbname", dbName);
            item.put("table_name2", table_name2);//首字母大写
            item.put("primaryKeyName", primaryKeyName.toString());
            cgHelper.generator("java_sync", table_name2 + "Sync.java", item);
        });
    }
}
